'use strict';

/* If your code uses external modules, create instances here. - 外部モジュールを使う場合はここでインスタンスを生成してください。
   e.g.  const request = require('xxxx');   // xxxx : external module - xxxxは外部モジュール */
const axios = require('axios');
const oauth2 = require('./OAuth2Util');
const { Base64 } = require('js-base64');

exports.handler = async event => {
  /* 
     Code for "warm start"
    The Riiiver system might call your Lambda function constantly to make it Warm Start.
    When the system called your function, the heavy modules are cached on the Lamba environment.
    The following code makes the (heavy) external module cached.
     ピースの実行速度高速化処理
    Lambda関数をウォームスタート状態にするために、このLambda関数は
    定期的にRiiiver System側からコールされることがあります。その際の処理をここで記述します。
    外部モジュールなどの重いインスタンスをキャッシュさせ、高速化します。*/
  if (event.call === 'lambda') {
    console.log('CALLED:LAMBDA');

    /* If you use the external modules, please code the following: - 外部モジュールを使う場合に記入してくだい。
       e.g.  if ( xxxx !== null ){} // xxxx: instance for the module.  -  xxxx : 生成したインスタンス */
    if (axios !== null && Base64 !== null && oauth2 !== null) { }
    return;
  }

  /* Start coding here - これ以降にコードを記入してください。*/

  console.log('sendGamil start: event = ', JSON.stringify(event));

  const responseData = {
    status: 200,
    body: {
      result: false
    }
  };

  let accessToken = null;
  if ('accessToken' in event.properties) {
    accessToken = event.properties.accessToken;
    console.log(`accessToken: ${accessToken}`);
  } else {
    console.log('user: ', JSON.stringify(event.userData));
    try {
      accessToken = await oauth2.getAccessToken(event.userData, event.serviceProxy);
    } catch (e) {
      return responseData;
    }
  }

  if (!accessToken) {
    console.log('Send Gmail error: accessToken is missing');
    return responseData;
  }

  const toEmail = event.properties.preferences.toEmail;
  const subject = event.properties.preferences.preSubject;
  const body = event.properties.preferences.preBody;
  const locationMessage = event.properties.input.message;
  const messageToUser = `${body}\n${locationMessage}`;

  const config = {
    url: 'https://www.googleapis.com/gmail/v1/users/me/messages/send?alt=json',
    method: 'post',
    headers: {
      'Authorization': `Bearer ${accessToken}`,
      'Content-Type': 'application/json',
    },
    data: {
      raw: Base64.encodeURI(formatEmail(toEmail, subject, messageToUser))
    }
  };

  return axios.request(config)
    .then(function (response) {
      if (response.status === 200) {
        responseData.body.result = true;
      } else {
        responseData.status = response.status || 500;
      }
      return responseData;
    }).catch(function (e) {
      console.log('Send Gmail error: ', JSON.stringify(e.message));
      responseData.status = 503;
      return responseData;
    })
};

function formatEmail(to, subject, body) {
  let headers = [];

  headers.push('Content-Type: text/plain; charset="utf-8"');
  headers.push('MIME-Version: 1.0');
  headers.push('Content-Transfer-Encoding: 7bit');
  headers.push(`To: ${to}`);
  headers.push(`Subject: =?utf-8?b?${encode_utf8(subject)}?=`);
  console.log(`=?utf-8?b?${encode_utf8(subject)}?=`);
  return [...headers, '', body].join('\r\n').toString();
}

function encode_utf8(s) {
  console.log(s);
  return Base64.encode(s);
}
